/*
 * WSaver module: 'pyrotechnics'.
 *
 * (w) 1998 by Eero Tamminen
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <Wlib.h>
#include "wsaver.h"

#define BORDER	50

#define POINTS	80	/* max. rockets/particles flying at the same time */
#define DIVIDE	18	/* divide rocket to approx. this many particles */

/* approx. 'ticks' before: */
#define NEXT	48	/* next rocket */
#define BURN	40	/* particles burn off */

#define BITS	6	/* fixed point accuracy */


/* flags */
#define PARTICLE	0
#define ROCKET		1
#define MOVED		2

typedef struct {
	short x, y;
	short dx, dy;
	unsigned type:2;
	unsigned timer:14;
} point_t;


static void shoot_rocket(point_t *point)
{
	int xsize = swidth - (2*BORDER);
	int ysize = sheight - BORDER;

	point->type = ROCKET;
	point->timer = (ysize>>1) - random() % (ysize>>2);
	/* horizontal [-0.5,0.5] and vertical [2,4] pixels */
	point->dx = random() % (1<<BITS) - (1<<(BITS-1));
	point->dy = -(random() % (2<<BITS) + (2<<BITS));
	point->x = (random() % xsize + BORDER) << BITS;
	point->y = ysize << BITS;
}

static void offset(short *x, short *y, short index, short max)
{
	/* sin(2*PI / 12 * index) * (1<<14) */
	static short sini[4] = { 0, 8192, 14189, 16384 };
	short div, xx, yy;

	div = index / 3;
	index = index - 3*div;
	switch(div) {
	case 3:
		xx = -sini[3-index];
		yy = -sini[index];
		break;
	case 2:
		xx = -sini[index];
		yy = sini[3-index];
		break;
	case 1:
		xx = sini[3-index];
		yy = sini[index];
		break;
	default:
		xx = sini[index];
		yy = -sini[3-index];
	}
	*x = ((long)xx * max) >> 14;
	*y = ((long)yy * max) >> 14;
}

static void explode(int *used, point_t *points, point_t *rocket)
{
	short x, y, dx, dy, pdx, pdy, burn, count, type = 0;
	point_t *new, *particle;

	dx = rocket->dx;
	dy = rocket->dy;
	x = rocket->x - dx;
	y = rocket->y - dx;
	burn = BURN + random() % (BURN/2);
	if (random() & 1) {
		count = 18;
		type = 1;
	} else {	
		count = DIVIDE + random() % (DIVIDE/2);
	}
	if (*used + count > POINTS) {
		count = POINTS - *used;
		type = 0;
	}
	new = &points[*used];
	*used += count;

	while(--count >= 0) {
		particle = new++;
		particle->type = PARTICLE;
		particle->timer = burn;

		if (type) {
			/* round */
			if (count >= 12) {
				/* inner ring: 6 */
				offset(&pdx, &pdy, (count%6)<<1, 1<<(BITS-1));
				particle->dx = dx + pdx;
				particle->dy = dy + pdy;
				particle->x = x;
				particle->y = y;
				particle->timer--;
				continue;
			}
			/* outer ring: 12 */
			offset(&pdx, &pdy, count, 1<<BITS);
		} else {
			/* randomized */
			pdx = random() % (2<<BITS) - (1<<BITS);
			pdy = random() % (2<<BITS) - (1<<BITS);
		}
		particle->dx = dx + pdx;
		particle->dy = dy + pdy;
		particle->x = x + 2*pdx;
		particle->y = y + 2*pdy;
	}
}

void save_pyro(void)
{
	/* used is index to first free point */
	static point_t points[POINTS];
	int idx, used = 0, timer = 1;
	short oldx, oldy;
	point_t *current;
	WEVENT *ev;

	for (;;) {
		if (!--timer) {
			timer = random() % NEXT + NEXT/2;
			if (used < POINTS) {
				shoot_rocket(&points[used++]);
			}
		}

		w_setmode(win, M_DRAW);
		/* re/move rockets/particles */
		for (idx = 0; idx < used; idx++) {

			current = &points[idx];
			oldx = current->x >> BITS;
			oldy = current->y >> BITS;

			current->x += current->dx;
			current->y += current->dy;

			/* timer / gravity */
			current->timer -= 1;
			current->dy += 1;

			if ((current->x >> BITS) != oldx ||
			    (current->y >> BITS) != oldy ||
			    !current->timer) {
				/* w_pbox(win, oldx-1, oldy-1, 3, 3); */
				w_pcircle(win, oldx, oldy, 2);

				if (current->timer) {
					current->type |= MOVED;
					continue;
				}

				/* timer expired */
				if (current->type & ROCKET) {
					/* create new points */
					explode(&used, points, current);
				}
				/* move latest to current */
				if (--used > idx) {
					*current = points[used];
					idx--;
				}
			} else {
				current->type &= ~MOVED;
			}
		}

		w_setmode(win, M_CLEAR);
		/* redraw rockets/particles */
		for (idx = 0; idx < used; idx++) {

			current = &points[idx];
			if (current->type & MOVED) {
				oldx = current->x >> BITS;
				oldy = current->y >> BITS;
				/* w_pbox(win, oldx-1, oldy-1, 3, 3); */
				w_pcircle(win, oldx, oldy, 2);
			}
		}

		if ((ev = w_queryevent(NULL, NULL, NULL, timeout))) {
			switch (ev->type) {

				case EVENT_GADGET:
					if (ev->key == GADGET_EXIT) {
						w_exit();
						exit(0);
					}
					break;

				default:
					return;
			}
		}
	}
}
