/*
 * lib/settings.c, part of Wlib, part of W
 * (C) 94-03/96 by Torsten Scherer (TeSche)
 * itschere@techfak.uni-bielefeld.de
 *
 * Changes:
 * 1/98 ++eero, added w_setpatterndata(), w_setlinewidth()
 */

#include <stdio.h>
#include "Wlib.h"
#include "proto.h"


short w_settextstyle(WWIN *win, short flags)
{
  char *cptr;
  STEXTSTYLEP paket;
  short old;

  TRACESTART();

  if ((cptr = _check_window(win))) {
    TRACEPRINT(("w_settextstyle(%p,%i) -> %s\n", win, flags, cptr));
    TRACEEND();
    return -1;
  }

  paket.len = sizeof(STEXTSTYLEP);
  paket.type = PAK_STEXTSTYLE;
  paket.handle = htons(win->handle);
  old = win->textstyle;
  paket.flags = htons(win->textstyle = flags & F_STYLEMASK);
  _send_paket((PAKET *)&paket);

  TRACEPRINT(("w_settextstyle(%p,%i)\n", win, flags));

  TRACEEND();
  return old;
}


short w_setsaver(short seconds)
{
  SSAVERP paket;
  short ret;

  TRACESTART();

  paket.len = sizeof(SSAVERP);
  paket.type = PAK_SSAVER;
  paket.seconds = htons(seconds);
  _send_paket((PAKET *)&paket);

  ret =  ntohs(((SRETP *)_wait4paket(PAK_SRET))->ret);

  TRACEPRINT(("w_setsaver(%i) -> %i\n", seconds, ret));
  TRACEEND();

  return ret;
}


ushort w_setpattern(WWIN *win, ushort pattern)
{
  char *cptr;
  SPATTERNP paket;
  ushort old;

  TRACESTART();

  if ((cptr = _check_window(win))) {
    TRACEPRINT(("w_setpattern(%p,0x%04x) -> %s\n",
		win, (unsigned int)pattern, cptr));
    TRACEEND();
    return 0;
  }

  paket.len = sizeof(SPATTERNP);
  paket.type = PAK_SPATTERN;
  paket.handle = htons(win->handle);
  old = win->pattern;
  paket.pattern = htons(win->pattern = pattern);     /* signed vs. unsigned? */
  _send_paket((PAKET *)&paket);

  TRACEPRINT(("w_setpattern(%p,0x%04x) -> 0x%04x\n",
	      win, (unsigned int)pattern, old));
  TRACEEND();

  return old;
}


ushort w_setpatterndata(WWIN *win, ushort *data)
{
  char *cptr;
  SPATTERNDATAP paket;
  ushort old;

  TRACESTART();

  if ((cptr = _check_window(win))) {
    TRACEPRINT(("w_setpatterndata(%p,%p) -> %s\n", win, data, cptr));
    TRACEEND();
    return -1;
  }

  paket.len = sizeof(SPATTERNDATAP);
  paket.type = PAK_SPATTERNDATA;
  paket.handle = htons(win->handle);
  memcpy(paket.data, data, sizeof(paket.data));
  _send_paket((PAKET *)&paket);

  old = win->pattern;
  win->pattern = W_USERPATTERN;

  TRACEPRINT(("w_setpatterndata(%p,%p) -> 0x%04x\n", win, data, old));
  TRACEEND();
  return old;
}


short w_setmode(WWIN *win, short mode)
{
  char *cptr;
  SMODEP paket;
  short old;

  TRACESTART();

  if ((cptr = _check_window(win))) {
    TRACEPRINT(("w_setmode(%p,%i) -> %s\n", win, mode, cptr));
    TRACEEND();
    return -1;
  }

  paket.len = sizeof(SMODEP);
  paket.type = PAK_SMODE;
  paket.handle = htons(win->handle);
  old = win->drawmode;
  paket.mode = htons(win->drawmode = mode);
  _send_paket((PAKET *)&paket);

  TRACEPRINT(("w_setmode(%p,%i) -> %i\n", win, mode, old));
  TRACEEND();

  return old;
}


short w_setlinewidth(WWIN *win, short width)
{
  char *cptr;
  SMODEP paket;
  short old;

  TRACESTART();

  if ((cptr = _check_window(win))) {
    TRACEPRINT(("w_setlinewidth(%p,%i) -> %s\n", win, width, cptr));
    TRACEEND();
    return -1;
  }

  if (width < 0) {
    TRACEPRINT(("w_setlinewidth(%p,%i) -> invalid value\n", win, width));
    TRACEEND();
    return win->linewidth;
  }

  paket.len = sizeof(SMODEP);
  paket.type = PAK_SLINEWIDTH;
  paket.handle = htons(win->handle);
  old = win->linewidth;
  paket.mode = htons(win->linewidth = width);
  _send_paket((PAKET *)&paket);

  TRACEPRINT(("w_setlinewidth(%p,%i) -> %i\n", win, width, old));
  TRACEEND();

  return old;
}
