/*
 * lib/winutil.c, part of Wlib, part of W
 * (C) 1994,95,96 by Torsten Scherer (TeSche)
 * itschere@techfak.uni-bielefeld.de
 *
 * 2/98, ++eero:
 * - added window construction from ID (win->handle has a value).  These
 *   windows can't be moved, resized, deleted nor receive events, just be
 *   inherited.
 *
 * 3/98, jps:
 * - added w_getmousepointer and w_setmousepointer
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Wlib.h"
#include "proto.h"


ulong w_winID(WWIN *win)
{
  return win->handle;
}


WWIN *w_winFromID(ulong id)
{
  WWIN *win;

  if (!(win = calloc(1, sizeof(WWIN)))) {
    return NULL;
  }
  win->magic = MAGIC_W;
  win->type = WWIN_FAKE;
  win->handle = id;

  w_querywinsize(win, 0, &win->width, &win->height);
  return win;
}


short w_getmousepointer (WWIN *win)
{
  char *cptr;
  GMOUSEP paket;
  short ret;

  TRACESTART ();

  if ((cptr = _check_window(win))) {
    TRACEPRINT(("w_getmousepointer(%p) -> NULL\n", win));
    TRACEEND();
    return -1;
  }

  paket.len = sizeof(GMOUSEP);
  paket.type = PAK_GMOUSE;
  paket.handle = htons(win->handle);
  _send_paket((PAKET *)&paket);

  ret = ntohs(((SRETP *)_wait4paket(PAK_SRET))->ret);

  TRACEPRINT(("w_getmousepointer(%p) -> %i\n", win, ret));

  TRACEEND();

  return ret;
}


short w_setmousepointer (WWIN *win, short type, WMOUSE *data)
{
  char *cptr;
  SMOUSEP paket;

  TRACESTART ();

  if ((cptr = _check_window(win))) {
    TRACEPRINT(("w_setmousepointer(%p,%i,%p) -> %s\n", win, type, data, cptr));
    TRACEEND();
    return -1;
  }

  paket.len = sizeof(SMOUSEP);
  paket.type = PAK_SMOUSE;
  paket.handle = htons(win->handle);
  paket.mtype = htons(type);
  /* not a predefined one? */
  if (type >= MOUSE_USER) {
    if (!data) {
      TRACEPRINT(("w_setmousepointer(%p,%i,%p) -> no data\n", win, type, data));
      TRACEEND();
      return -1;
    }
    paket.xoff = htons(data->xDrawOffset);
    paket.yoff = htons(data->yDrawOffset);
    memcpy(paket.mask, data->mask, sizeof(paket.mask));
    memcpy(paket.icon, data->icon, sizeof(paket.icon));
  }
  _send_paket((PAKET *)&paket);

  TRACEPRINT(("w_setmousepointer(%p,%i,%p) -> 0\n", win, type, data));
  TRACEEND();

  /* Must flush so mouse pointer will change immediately.
   */
  w_flush();

  return 0;
}
