/*
 * server/client_others.c, part of W
 * (C) 1994,95,96 by Torsten Scherer (TeSche)
 * itschere@techfak.uni-bielefeld.de
 *
 * these are other functions. in fact they're only *here* to
 * keep the source somehow structured and small for each part
 *
 * 1/98 ++eero:
 * - moved all the setting (mode, color etc) functions from client_noret.c
 * to here as that file was getting a bit big. Added line width and
 * pattern data setting.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#ifdef __MINT__
# include <mintbind.h>
#endif

#include "config.h"
#include "types.h"
#include "pakets.h"
#include "proto.h"
#include "window.h"
#include "rect.h"	/* for ellipse/bezier */


/*
 *
 */

short client_status(CLIENT *cptr, CLIENT *clients, short index, STATUS *status)
{
  short i;
  CLIENT *ptr;

  if (index < 0) {

    i = 0;
    ptr = clients;
    while (ptr) {
      i++;
      ptr = ptr->next;
    }

    status->ip_addr = ntohl(0);
    status->pakets = ntohl(glob_pakets);
    status->bytes = ntohl(glob_bytes);
    status->openWin = ntohs(globOpenWindows);
    status->totalWin = ntohs(globTotalWindows);

    return i;
  }

  /* try to get a specific client */

  ptr = clients;
  while (ptr && (index-- > 0)) {
    ptr = ptr->next;
  }

  if (!ptr) {
    return -1;
  }

  status->ip_addr = ntohl(ptr->raddr);
  status->pakets = ntohl(ptr->pakets);
  status->bytes = ntohl(ptr->bytes);
  status->openWin = ntohs(ptr->openWindows);
  status->totalWin = ntohs(ptr->totalWindows);

  return 0;
}


void client_setmousepointer (CLIENT *cptr, ushort handle, short type,
                             short xoff, short yoff, ushort *mask, ushort *icon)
{
  WINDOW *win;

  if (!(win = windowLookup (handle))) {
    return;
  }

  mouse_hide ();

  if (win->mdata) {
    free(win->mdata);
    win->mtype = MOUSE_ARROW;
  }

  if (type == MOUSE_USER) {
    if (!(win->mdata = malloc(sizeof(WMOUSE)))) {
      return;
    }
    win->mdata->xDrawOffset = xoff;
    win->mdata->yDrawOffset = yoff;
    memcpy(win->mdata->mask, mask, sizeof(win->mdata->mask));
    memcpy(win->mdata->icon, icon, sizeof(win->mdata->icon));
  } else {
    win->mdata = NULL;
  }

  win->mtype = type;
  mouse_show ();
}


short client_getmousepointer (CLIENT *cptr, ushort handle)
{
  WINDOW *win;

  if (!(win = windowLookup (handle))) {
    return -1;
  }

  return win->mtype;
}


void client_settitle(CLIENT *cptr,
		     ushort handle,
		     char *s)
{
  WINDOW *win;
  static GCONTEXT gc = {M_DRAW, 1, F_NORMAL,
			  &glob_font[TITLEFONT], DefaultPattern,
			  1, 0,
			  {0xffff,0x0,0x0,0x0,0x0,0x0,0x0,0x0},
			  {0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0}};

  if (!(win = windowLookup (handle)))
    return;

  if (!(win->flags & W_TITLE))
    return;

  strncpy (win->title, s, MAXTITLE);

  if (win->flags & W_CONTAINER)
    return;

  gc0 = &gc;

#ifndef REFRESH
  if (win->is_open && !win->is_hidden) {
    if (mouse_rcintersect(win->pos.x0 + win->area[AREA_TITLE].x0,
			  win->pos.y0 + win->area[AREA_TITLE].y0,
			  win->area[AREA_TITLE].w,
			  win->area[AREA_TITLE].h)) {
      mouse_hide();
    }
    clip0 = &win->pos;
    (*glob_screen->dpbox)(&glob_screen->bm,
			  win->pos.x0 + win->area[AREA_TITLE].x0,
			  win->pos.y0 + win->area[AREA_TITLE].y0,
			  win->area[AREA_TITLE].w,
			  win->area[AREA_TITLE].h);
    (*glob_screen->prints)(&glob_screen->bm,
			   win->pos.x0 + win->area[AREA_TITLE].x0,
			   win->pos.y0 + win->area[AREA_TITLE].y0,
			   s);
    win->is_dirty = 1;
  } else
#endif
  {
    clip0 = &win->area[AREA_TITLE];
    (*glob_screen->dpbox)(&win->bitmap,
			  win->area[AREA_TITLE].x0,
			  win->area[AREA_TITLE].y0,
			  win->area[AREA_TITLE].w,
			  win->area[AREA_TITLE].h);
    (*glob_screen->prints)(&win->bitmap,
			   win->area[AREA_TITLE].x0,
			   win->area[AREA_TITLE].y0,
			   s);
    if (win->is_open) {
      rectUpdateDirty(&win->dirty,
		      win->area[AREA_TITLE].x0,
		      win->area[AREA_TITLE].y0,
		      win->area[AREA_TITLE].w,
		      win->area[AREA_TITLE].h);
    }
  }

  win->has_title = 1;
}


void client_beep(CLIENT *cptr)
{
#ifdef __MINT__
  Bconout(2, 7);
#else
  fprintf(stdout, "\007");
  fflush(stdout);
#endif
}


void client_setmode(CLIENT *cptr, ushort handle, short mode)
{
  WINDOW *win;

  if ((win = windowLookup(handle))) {
    win->gc.drawmode = mode;
  }
}


void client_setlinewidth(CLIENT *cptr, ushort handle, short width)
{
  WINDOW *win;

  if ((win = windowLookup(handle))) {
    win->gc.linewidth = width;
  }
}


void client_setfont(CLIENT *cptr, ushort handle, short fonthandle)
{
  WINDOW *win;
  LIST *font;
  int idx;

  if ((fonthandle < 0) || (fonthandle >= MAXFONTS)) {
    return;
  }

  if (glob_font[fonthandle].numused && (win = windowLookup(handle))) {
    idx = -1;
    for (font = cptr->font; font; font = font->next) {
      if (font->handle == fonthandle) {
        win->gc.font = &glob_font[fonthandle];
	break;
      }
    }
    /* else client had not loaded that font */
  }
}


void client_settextstyle(CLIENT *cptr, ushort handle, short flags)
{
  WINDOW *win;

  if ((win = windowLookup(handle)))
    win->gc.textstyle = flags & F_STYLEMASK;
}


void client_setpatterndata(CLIENT *cptr, ushort handle, ushort *data)
{
  WINDOW *win;

  if ((win = windowLookup(handle))) {
    memcpy(win->patbuf, data, sizeof(win->patbuf));
  }
}


/* Manufacture 16x16 pattern according to the pattern id. */

void client_setpattern(CLIENT *cptr, ushort handle, ushort id)
{
  static ushort DM64[8][8] = {			/* courtesy of Kay... */
    {  0, 32,  8, 40,  2, 34, 10, 42 },
    { 48, 16, 56, 24, 50, 18, 58, 26 },
    { 12, 44,  4, 36, 14, 46,  6, 38 },
    { 60, 28, 52, 20, 62, 30, 54, 22 },
    {  3, 35, 11, 43,  1, 33,  9, 41 },
    { 51, 19, 59, 27, 49, 17, 57, 25 },
    { 15, 47,  7, 39, 13, 45,  5, 37 },
    { 63, 31, 55, 23, 61, 29, 53, 21 }
  };
  WINDOW *win;
  ushort patt, *patbuf;
  ushort *intensity;
  int i;

  if (!(win = windowLookup(handle)))
    return;

  patbuf = win->patbuf;

  /* grayscale patterns */
  if(id <= MAX_GRAYSCALES)
  {
    for (i=0; i < 8; i++)
    {
      patt = 0;
      intensity = DM64[i];
      /* matrix values [0-63] -> id 0=black, 64=white */
      if(id <= *intensity++) patt  = 0x0101;
      if(id <= *intensity++) patt |= 0x0202;
      if(id <= *intensity++) patt |= 0x0404;
      if(id <= *intensity++) patt |= 0x0808;
      if(id <= *intensity++) patt |= 0x1010;
      if(id <= *intensity++) patt |= 0x2020;
      if(id <= *intensity++) patt |= 0x4040;
      if(id <= *intensity++) patt |= 0x8080;
      patbuf[8] = patt;
      *patbuf++ = patt;
    }
    return;
  }

  /* line patterns (rolled) */
  if(id >= 0xff)
  {
    memset(patbuf, 0, sizeof(win->patbuf));

    /* roll right */
    if (id & 0x8100) {
      for(i = 0; i < 16; i++) {
	*patbuf++ |= id;
	id = ((id & 1) << 15) | (id >> 1);
      }
      patbuf -= 16;
    }
    /* roll left */
    if (id & 0x300) {
      for(i = 0; i < 16; i++) {
	*patbuf++ |= id;
	id = ((id & 0x8000) >> 15) | (id << 1);
      }
    }
    /* pattern is a hatch if 1st upper byte bit is set */
    return;
  }

  /* misc patterns (eg. images) */
  switch(id)
  {
    case W_PATTERN:
      memcpy(win->patbuf, BackGround, sizeof(win->patbuf));
      break;
    default:
      memcpy(win->patbuf, DefaultPattern, sizeof(win->patbuf));
  }
}


/*
 * some color functions
 */

short clientFreeColor(CLIENT *cptr, ushort handle, short color)
{
  WINDOW *win;

  if (!(win = windowLookup (handle)))
    return -1;

  colorFreeColor (win, color);

  return 0;
}


void clientChangeColor(CLIENT *cptr, ushort handle, short color,
		       ushort red, ushort green, ushort blue)
{
  WINDOW *win;

  if (!(win = windowLookup (handle)))
    return;

  colorChangeColor (win, color, red & 255, green & 255, blue & 255);

  if (win == glob_activewindow)
    colorSetColorTable (win->colTab, 1);
}


void clientSetColor (CLIENT *cptr, ushort handle, short color)
{
  WINDOW *win;

  if (!(win = windowLookup (handle)))
    return;

  switch (glob_pakettype) {
    case PAK_SETFGCOL:
      colorSetFGColor (win, color);
      break;
    case PAK_SETBGCOL:
      colorSetBGColor (win, color);
  }
}

