/*
 * server/graph/sun.c, part of W
 * (C) 94-07/96 TeSche (Torsten Scherer)
 * itschere@techfak.uni-bielefeld.de
 *
 * init code for FreeBSD 3.0
 * (C) 03-98 Andrzej Bialecki <abial@nask.pl>
 */

#ifdef __FreeBSD__

#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <machine/console.h>
#include <vgl.h>
#include "../config.h"
#include "../types.h"
#include "gproto.h"
#ifdef DIRECT8
#include "direct8/direct8.h"
#else
#include "monochrome/bmono.h"
#endif
#include "generic/generic.h"

#ifdef DIRECT8
void freebsdPutCmap (COLORTABLE *colTab)
{
	VGLSetPalette(colTab->red, colTab->green, colTab->blue);
	return;
}

SCREEN freebsd_screen = {
  {}, freebsdPutCmap, NULL, direct8_mouseShow, direct8_mouseHide,
   direct8_createbm, direct8_bitblk, direct8_scroll,
   direct8_prints, direct8_test,
   direct8_plot, direct8_line, direct8_hline, direct8_vline,
   generic_box, generic_pbox, generic_circ, generic_pcirc,
   generic_ellipse, generic_pellipse, generic_arc, generic_pie,
   generic_poly, generic_ppoly, generic_bezier, direct8_dplot,
   direct8_dline, direct8_dvline, direct8_dhline, generic_dbox,
   generic_dpbox, generic_dcirc, generic_dpcirc, generic_dellipse,
   generic_dpellipse, generic_darc, generic_dpie, generic_dpoly,
   generic_dppoly, generic_bezier
};
#else
void freebsdPutCmap (COLORTABLE *colTab)
{
	return;
}

SCREEN freebsd_screen = {
  {}, freebsdPutCmap, NULL, bmono_mouseShow, bmono_mouseHide,
   bmono_createbm, bmono_bitblk, bmono_scroll,
   bmono_prints, bmono_test,
   bmono_plot, bmono_line, bmono_hline, bmono_vline,
   generic_box, generic_pbox, generic_circ, generic_pcirc,
   generic_ellipse, generic_pellipse, generic_arc, generic_pie,
   generic_poly, generic_ppoly, generic_bezier, bmono_dplot,
   bmono_dline, bmono_dvline, bmono_dhline, generic_dbox,
   generic_dpbox, generic_dcirc, generic_dpcirc, generic_dellipse,
   generic_dpellipse, generic_darc, generic_dpie, generic_dpoly,
   generic_dppoly, generic_bezier
};
#endif

SCREEN *freebsd_init (void)
{
  int size;
  int debug=1;

#ifdef DIRECT8
  if(VGLInit(SW_VGA_CG320)) {
#else
  if(VGLInit(SW_BG640x480)) {
#endif
	fprintf(stderr,"freebsd_init(): Unsupported mode!\n");
	return NULL;
  }

  freebsd_screen.bm.width = VGLDisplay->Xsize;
  freebsd_screen.bm.height = VGLDisplay->Ysize;
#ifdef DIRECT8
  freebsd_screen.bm.type = BM_DIRECT8;
  freebsd_screen.bm.unitsize = 1;
  freebsd_screen.bm.upl = VGLDisplay->Xsize;
  freebsd_screen.bm.planes = 8;
#else
  freebsd_screen.bm.type = BM_PACKEDMONO;
  freebsd_screen.bm.unitsize = 4;
  freebsd_screen.bm.upl = VGLDisplay->Xsize>>5;
  freebsd_screen.bm.planes = 1;
#endif
  freebsd_screen.bm.data = VGLDisplay->Bitmap;

  return &freebsd_screen;
}

#endif
