/*
 * generic/prints.h, part of W
 * (C) 94-04/96 by Torsten Scherer (TeSche)
 * itschere@techfak.uni-bielefeld.de
 *
 * generic string output routines using driver specific character routines.
 *
 * Changes:
 *
 * - Split TeSche's and Kay's text.c code into generic and driver specific
 *   parts and moved clipping to prints.  I have no-styles character output
 *   as a separate routine because terminal is most used application and
 *   therefore text speed is crucial.
 * - Implemented underline with hline().
 *   ++eero 11/96
 * - This is now a generic include file so that the driver specific
 *   character output routines can be inlined for it. 2/98 ++eero
 *
 * NOTES:
 * - This needs a FUNCTION macro on the file including this that which will
 *   expand the canonical FUNCTION(normalc), FUNCTION(styles) and
 *   FUNCTION(prints) to their driver specific names.
 */

/* 'italics' char area isn't rectangular so we'll need this */
static int dirtyClip;


/* Clip a character.  Mainly used for italics at window sides, although due
 * bitblk rectangle overlap the character at the right border will then
 * clear some pixels off the previous character.  With other styles the
 * rectangles won't overlap.
 */
static void FUNCTION(clipchar) (destbm, x0, y0, width, height, c, effects)
     BITMAP *destbm;
     long x0;
     long y0;
     short width;
     short height;
     uchar c;
     ushort effects;
{
  REC *save_clip1;
  ushort slant = 0;

  if (effects & F_ITALIC) {

     if (dirtyClip) {
       int bg = 0;
       if (theScreen->bm.planes > 1) {
	 bg = gc0->bgCol;
       }
       memset(bitmapTmp.data, bg, bitmapTmp.upl * bitmapTmp.unitsize * height);
       dirtyClip = 0;
     }

     slant = gc0->font->slant_size;
     width += slant;
  }

  if (width > bitmapTmp.width || height > bitmapTmp.height) {
    return;
  }

  /* how I'll say that here we don't need inlining? */
  if (effects & ~F_UNDERLINE) {
    FUNCTION(stylec)(&bitmapTmp, slant, 0, c);
  } else {
    FUNCTION(normalc)(&bitmapTmp, 0, 0, c);
  }

  save_clip1 = clip1;
  clip1 = clip0;
  clip0 = &clipTmp;
  (*theScreen->bitblk)(&bitmapTmp, 0, 0, width, height, destbm, x0, y0);
  clip0 = clip1;
  clip1 = save_clip1;
}


void FUNCTION(prints) (bm, x0, y0, s)
     BITMAP *bm;
     long x0;
     long y0;
     uchar *s;
{
  FONT  *font = gc0->font;
  ushort effects = gc0->textstyle;
  uchar *widths = font->widths;
  short height = font->hdr.height;
  short slant, ul_x, ul_w, width = 0, count = 0;
  uchar *end;

  /* string lenght & width */
  end = s;
  while (*end) {
    width += widths[*end++];
    count++;
  }

  /* for underline */
  ul_w = width;
  ul_x = x0;

  /* only necessary and allowed (readable) styles */
/*  effects &= font->hdr.effect_mask; */
  font->effects = effects;

  /* italic correction */
  if (effects & F_ITALIC) {
    /* check clipping against leftmost position */
    x0 -= font->slant_offset;
    slant = font->slant_size;
    width += slant;
  } else {
    slant = 0;
  }

  /* 
   * Clip necessary chars
   */
  if (clip0) {

    dirtyClip = 1;

    /* find the rightmost visible char and clip it if necessary */
    if (x0 + width > clip0->x1) {
      while (count) {
	count--;
	width -= widths[*--end];
	if (x0 + width <= clip0->x1) {
 	  FUNCTION(clipchar) (bm, x0 + width, y0, widths[*end], height, *end, effects);
	  break;
        }
      }
    }

    /* find the leftmost visible char and clip it if necessary */
    if (x0 < clip0->x0) {
      while(count) {
        count--;
        x0 += widths[*s++];
        if (x0 >= clip0->x0) {
	  char c = *(s-1);
	  FUNCTION(clipchar) (bm, x0 - widths[c], y0, widths[c], height, c, effects);
	  break;
        }
      }
    }
    if (count < 1) {
      goto bottomline;
    }

    /* clip rest of the chars in vertical direction if necessary */
    if (y0 < clip0->y0 || y0 + height - 1 > clip0->y1) {
      if (y0 > clip0->y1 || y0 + height <= clip0->y0) {
        return;
      }
      while (count--) {
	FUNCTION(clipchar) (bm, x0, y0, widths[*s], height, *s, effects);
	x0 += widths[*s++];
      }
      goto bottomline;
    }
  }


  /*
   * rest of the string doesn't need clipping...
   */

  if (effects & ~F_UNDERLINE) {
    x0 += slant;
    while (count--) {
      FUNCTION(stylec) (bm, x0, y0, *s);
      x0 += widths[*s++];
    }
  } else {
    while (count--) {
      FUNCTION(normalc) (bm, x0, y0, *s);
      x0 += widths[*s++];
    }
  }


bottomline:

  if (effects & F_UNDERLINE) {
    short old_mode = gc0->drawmode;

    if ((effects | font->hdr.styles) & F_REVERSE) {
      gc0->drawmode = M_CLEAR;
    } else {
      gc0->drawmode = M_DRAW;
    }

    /* this check is needed for e.g. console */
    if (font->hdr.baseline + 2 < height) {
      y0 += font->hdr.baseline + 2;
    } else {
      y0 += height - 1;
    }
    (*theScreen->hline)(bm, ul_x, y0, ul_x + ul_w - 1);
    gc0->drawmode = old_mode;
  }
}
